/* **********************************************************************************
 *
 * Copyright (c) TMF.AscendedThemeKit.Unthemed.NET Project. All rights reserved.
 *
 * This source code is subject to terms and conditions of the Shared Source License
 * for TMF.AscendedThemeKit.Unthemed. A copy of the license can be found in the License.html file
 * at the root of this distribution. If you can not locate the Shared Source License
 * for TMF.AscendedThemeKit.Unthemed, please send an email to ascend_adm@hotmail.com.
 * By using this source code in any fashion, you are agreeing to be bound by
 * the terms of the Shared Source License for TMF.AscendedThemeKit.Unthemed.NET.
 *
 * You must not remove this notice, or any other, from this software.
 *
 * **********************************************************************************/

using System;
using System.Collections.Generic;
using System.ComponentModel.Design;
using System.Text;
using System.Windows.Forms.Design;
using TMF.AscendedThemeKit.Unthemed.Resources;

namespace TMF.AscendedThemeKit.Unthemed.Windows.Forms.Design
{
    /// <summary>
    /// Extends the design mode behavior of the ControlDesigner.
    /// </summary>
    /// <remarks>
    /// <para>
    /// ControlDesigner provides a base class for designers of components that derive from Control. In addition to the methods and functionality inherited from the ComponentDesigner class, ControlDesigner provides additional methods to support extending and altering the behavior of an associated Control at design time.
    /// </para>
    /// <para>
    /// You can associate a designer with a type using a DesignerAttribute. For an overview of customizing design time behavior.
    /// </para>
    /// </remarks>
    public class GradientSplitterDesigner : ControlDesigner
    {
        private DesignerActionListCollection _actionLists;

        /// <summary>
        /// Gets the design-time action lists supported by the component associated with the designer.
        /// </summary>
        /// <value>
        /// The design-time action lists supported by the component associated with the designer. 
        /// </value>
        public override DesignerActionListCollection ActionLists
        {
            get
            {
                if (this._actionLists == null)
                {
                    this._actionLists = new DesignerActionListCollection();
                    this._actionLists.Add(new GradientSplitterActionList((GradientSplitter)Control));

                }

                return this._actionLists;

            }

        }

    }

}
